import re
import json
import logging
from contextlib import AsyncExitStack
from types import TracebackType
from typing import Any, ClassVar, Pattern, Self

import aiohttp

logger = logging.getLogger(__name__)


class FrigateConfig:
    _URL_ADDRESS_REGEX: ClassVar[Pattern[str]] = re.compile('(^|(?<=://)|(?<=@))[a-z0-9.\\-]+(:[0-9]+)?($|(?=/))')
    _WYZE_CAMERAS: ClassVar[dict[str, str]] = {'back_yard_cam': '192.168.0.202:554'}

    def __init__(self, frigate_base_url: str = 'http://frigate:5000') -> None:
        self._frigate_config_url = f'{frigate_base_url}/api/config'
        self._config = {}
        self._aiohttp_session: aiohttp.ClientSession
        self._async_exit_stack: AsyncExitStack

    async def __aenter__(self) -> Self:
        async with AsyncExitStack() as async_exit_stack:
            self._aiohttp_session = await async_exit_stack.enter_async_context(aiohttp.ClientSession(raise_for_status=True))
            self._async_exit_stack = async_exit_stack.pop_all()

            await self.refresh()
        return self

    async def __aexit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None) -> None:
        await self._async_exit_stack.aclose()

    async def refresh(self) -> None:
        logger.debug('Fetching Frigate config...')
        try:
            async with self._aiohttp_session.get(self._frigate_config_url) as response:
                self._config = json.loads(await response.read())
            logger.debug('Finished fetching Frigate config')
        except Exception as e:
            if self._config:
                logger.warning('Failed to fetch Frigate config, falling back to previous value', exc_info=e)
            else:
                raise

    @property
    def active_cameras(self) -> list[str]:
        if 'cameras' not in self._config:
            raise ValueError('Configuration not yet fetched from Frigate')

        return [camera for camera in self._config['cameras'] if self._config['cameras'][camera].get('enabled', True)]

    @property
    def active_camera_addresses(self) -> dict[str, str]:
        active_cameras = self.active_cameras
        return {camera: self._get_address_from_url(self._config['cameras'][camera]['ffmpeg']['inputs'][0]['path']) for camera in active_cameras}

    @classmethod
    def _get_address_from_url(cls, url: str) -> str:
        match = cls._URL_ADDRESS_REGEX.search(url.lower())
        if match is None:
            raise ValueError(f'Failed to retrieve address from {url=}')

        # Handle case of wyze-bridge and hardcode cameras
        if match.group().startswith('wyze-bridge'):
            wyze_camera = url.lower().rsplit('/', 1)[-1].replace('-', '_')
            return cls._WYZE_CAMERAS[wyze_camera]
        return match.group()
